module XRS

TRANSPARENCY = 0.25

FaceData2 = Struct.new("FaceData2", :face, :material, :back_material, :alpha, :balpha)

def XRS.xray_selection
	@model = Sketchup.active_model
	@tw = Sketchup.create_texture_writer
	selection = @model.selection
	if (selection.length == 0)	#no selection, just toggle textures
		return UI.beep unless @hsh_faces
		@xray = ! @xray
	else
		@hsh_faces = {}
		XRS.get_faces_at_level selection, {}
		return UI.beep unless @hsh_faces.length > 0
		@xray = true
		#XRS.find_texture_files
	end	
	
	@pbar = ProgressionBar.new @hsh_faces.length, "Faces"
	@model.start_operation "Xray faces"
		XRS.xray_faces
		selection.clear if (selection.length > 0)
	@model.commit_operation
end

def XRS.get_faces_at_level(entities, hcomp)
	entities.each do |e| 
		case e.typename
		when 'Group'
			XRS.get_faces_at_level e.entities, hcomp
		when 'ComponentInstance'
			edef = e.definition
			next if hcomp[edef.to_s]
			hcomp[edef.to_s] = edef
			XRS.get_faces_at_level edef.entities, hcomp
		when 'Face'
			fd = FaceData2.new
			fd.face = e
			fd.material = e.material
			fd.back_material = e.back_material
			@hsh_faces[e.to_s] = fd
		end
	end
end	

def XRS.material_exist?(name)
	@model.materials.each { |m| return m if m.name == name }
	return nil
end

def XRS.clone_material(face, front)
	mat = (front) ? face.material : face.back_material
	if (mat == nil)
		name = "whiteXX" + '___XR'
		newmat = XRS.material_exist? name
		unless newmat
			newmat = @model.materials.add name
			newmat.color = "white"
			newmat.alpha = TRANSPARENCY
		end	
	else
		name = mat.name + '___XR'
		newmat = XRS.material_exist? name
		unless newmat
			newmat = @model.materials.add name
			newmat.color = mat.color
			newmat.alpha = TRANSPARENCY
		end	
	end	
	newmat
end

def XRS.find_texture_files
	topdir = Sketchup.find_support_file "Materials"
	sub = Dir[topdir + "/*"]
	puts "topdir = #{topdir}"
	#puts "sub = #{sub}"
	sub.each do |d|
		puts "d = #{d}"
		lf = Dir[d + "/*.jpg"]
		lf.each do |f|
			puts "#{f.basename} --> f = #{f}"
		end
	end	
end

def XRS.xray_faces
	@hsh_faces.each do |index, fd|
		next unless fd.face.valid?
		face = fd.face
		if (@xray)
			face.material = XRS.clone_material(face, true)
			face.back_material = XRS.clone_material(face, false)
		else
			face.material = fd.material
			face.back_material = fd.back_material
		end
		@pbar.countage
	end
end


#--------------------------------------------------------------------------------------------------------------
# Class ProgressionBar: progress bar in the Sketchup Status text area
#--------------------------------------------------------------------------------------------------------------
class ProgressionBar

#Initialization of progress bar
def initialize(nbelts, label)
	@pb_nbelts = nbelts
	@pb_label = label
	@pb_progression = 0
	@pb_rangemax = 200
	@pb_range = 0
	@pb_time0 = Time.now	
end

#Increment the Progression Bar by <nb> steps
def countage(nb=1)
	@pb_progression += nb
	f = 100 * @pb_progression / @pb_nbelts
	percent = f.to_i
	if (percent != @pb_range)
		@pb_range = percent
		n = 1 + percent * @pb_rangemax / 100
		Sketchup::set_status_text "|" * n.to_i
	end	
	Sketchup.set_status_text @pb_label + " #{@pb_progression} / #{@pb_nbelts}", SB_VCB_LABEL
	Sketchup::set_status_text "#{@pb_range}%  -  #{sprintf "%4.2f", Time.now - @pb_time0} sec", SB_VCB_VALUE
end

end #class ProgressionBar

def XRS.set_command
	iconpath = "IconXraySelection.png"
	menutool = UI.menu "Plugins"
	menutool.add_separator
	@tlb = nil
	
	#Joint Push Pull Command
	cmd = UI::Command.new("Xray Selection (toggle)") { XRS.xray_selection }
	menutool.add_item cmd
	tooltip = "Xray the selection and then toggle between Xray and plain texture"
	
	cmd.status_bar_text = tooltip
	icon = Sketchup.find_support_file iconpath, "Plugins/"
	if (icon)
		cmd.tooltip = tooltip
		cmd.small_icon = cmd.large_icon = icon
		@tlb = UI::Toolbar.new "XRAY" unless @tlb
		@tlb.add_item cmd
	end	
	@tlb.show if @tlb
end

unless $XRS____loaded
	XRS.set_command
	$XRS____loaded = true
end
	
end #module XRS
